from django_codemod.constants import DJANGO_1_8, DJANGO_1_11, DJANGO_3_1
from django_codemod.visitors.base import BaseRenameTransformer


class QueryEmptyResultSetTransformer(BaseRenameTransformer):
    """Replace `django.db.models.query.EmptyResultSet` compatibility import."""

    deprecated_in = DJANGO_1_11
    removed_in = DJANGO_3_1
    rename_from = "django.db.models.query.EmptyResultSet"
    rename_to = "django.core.exceptions.EmptyResultSet"


class SqlEmptyResultSetTransformer(BaseRenameTransformer):
    """Replace `django.db.models.sql.EmptyResultSet` compatibility import."""

    deprecated_in = DJANGO_1_11
    removed_in = DJANGO_3_1
    rename_from = "django.db.models.sql.EmptyResultSet"
    rename_to = "django.core.exceptions.EmptyResultSet"


class DatastructuresEmptyResultSetTransformer(BaseRenameTransformer):
    """
    Replace `django.db.models.sql.datastructures.EmptyResultSet` compatibility
    import.
    """

    deprecated_in = DJANGO_1_11
    removed_in = DJANGO_3_1
    rename_from = "django.db.models.sql.datastructures.EmptyResultSet"
    rename_to = "django.core.exceptions.EmptyResultSet"


class FieldDoesNotExistTransformer(BaseRenameTransformer):
    """Replace `django.db.models.fields.FieldDoesNotExist` compatibility import."""

    deprecated_in = DJANGO_1_8
    removed_in = DJANGO_3_1
    rename_from = "django.db.models.fields.FieldDoesNotExist"
    rename_to = "django.core.exceptions.FieldDoesNotExist"



from typing import Optional, Sequence

from libcst import Arg, Call, FunctionDef, Param
from libcst import matchers as m
from libcst import parse_expression, parse_statement


def find_keyword_arg(args: Sequence[Arg], keyword_name: str) -> Optional[Arg]:
    """Find a kwarg among a sequence of arguments."""
    matcher = m.Arg(keyword=m.Name(keyword_name))
    for arg in args:
        if m.matches(arg, matcher):
            return arg
    return None


def parse_arg(arg_str: str) -> Arg:
    """Build a `Arg` instance based on its string representation.

    Instantiating it from scratch is cumbersome, this helper generates a
    function call with the given argument and extract it from the tree.
    """
    call_result = parse_expression(f"call({arg_str})")
    if isinstance(call_result, Call):
        return call_result.args[0]
    raise AssertionError(f"Unexpected type for: {call_result}")


def parse_param(arg_str: str) -> Param:
    """Build a `Param` instance based on its string representation.

    Instantiating it from scratch is cumbersome, this helper generates a
    function definition with the given param and extract it from the tree.
    """
    statement = parse_statement(f"def fun({arg_str}):...")
    if isinstance(statement, FunctionDef):
        return statement.params.params[0]
    raise AssertionError(f"Unexpected type for: {statement}")



from django_codemod.constants import DJANGO_1_7, DJANGO_3_1
from django_codemod.visitors.base import BaseRenameTransformer


class BaseContextTransformer(BaseRenameTransformer):
    """Replace `django.template.base.BaseContext` compatibility import."""

    deprecated_in = DJANGO_1_7
    removed_in = DJANGO_3_1
    rename_from = "django.template.base.BaseContext"
    rename_to = "django.template.context.BaseContext"


class ContextTransformer(BaseRenameTransformer):
    """Replace `django.template.base.Context` compatibility import."""

    deprecated_in = DJANGO_1_7
    removed_in = DJANGO_3_1
    rename_from = "django.template.base.Context"
    rename_to = "django.template.context.Context"


class RequestContextTransformer(BaseRenameTransformer):
    """Replace `django.template.base.RequestContext` compatibility import."""

    deprecated_in = DJANGO_1_7
    removed_in = DJANGO_3_1
    rename_from = "django.template.base.RequestContext"
    rename_to = "django.template.context.RequestContext"


class ContextPopExceptionTransformer(BaseRenameTransformer):
    """Replace `django.template.base.ContextPopException` compatibility import."""

    deprecated_in = DJANGO_1_7
    removed_in = DJANGO_3_1
    rename_from = "django.template.base.ContextPopException"
    rename_to = "django.template.context.ContextPopException"



from django_codemod.constants import DJANGO_1_9, DJANGO_3_1
from django_codemod.visitors.base import BaseRenameTransformer


class PrettyNameTransformer(BaseRenameTransformer):
    """Replace `django.forms.forms.pretty_name` compatibility import."""

    deprecated_in = DJANGO_1_9
    removed_in = DJANGO_3_1
    rename_from = "django.forms.forms.pretty_name"
    rename_to = "django.forms.utils.pretty_name"


class BoundFieldTransformer(BaseRenameTransformer):
    """Replace `django.forms.forms.BoundField` compatibility import."""

    deprecated_in = DJANGO_1_9
    removed_in = DJANGO_3_1
    rename_from = "django.forms.forms.BoundField"
    rename_to = "django.forms.boundfield.BoundField"
